﻿using PhotoBrowser.Model;
using PhotoBrowser.Services;
using PhotoBrowser.ViewModel;
using System.Diagnostics;
using System.Threading;
using Windows.ApplicationModel.Activation;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace PhotoBrowser
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class ExtendedSplash : Page
    {
        private Frame rootFrame;
        private SynchronizationContext _uiContext;

        private string _activationTileId;
        private string _tileArguments;

        public ExtendedSplash(SplashScreen splashScreen,
            string activationTileID, string tileArguments)
        {
            this.InitializeComponent();

            _uiContext = SynchronizationContext.Current;

            _activationTileId = activationTileID;
            _tileArguments = tileArguments;

            if (splashScreen != null)
            {
                splashScreen.Dismissed += OnSplashScreenDismissed;

                Canvas.SetLeft(SplashScreenImage, 
                    splashScreen.ImageLocation.Left);

                Canvas.SetTop(SplashScreenImage, 
                    splashScreen.ImageLocation.Top);

                SplashScreenImage.Width = splashScreen.ImageLocation.Width;
                SplashScreenImage.Height = splashScreen.ImageLocation.Height;
                SplashScreenImage.Visibility = Visibility.Visible;

                ProgressContainer.Margin = new Thickness(
                    0,
                    splashScreen.ImageLocation.Top +
                    splashScreen.ImageLocation.Height,
                    0, 0);
            }
            rootFrame = new Frame();

            Debug.WriteLine("Construction Thread ID: " + System.Environment.CurrentManagedThreadId);
        }

        async void OnSplashScreenDismissed(SplashScreen sender, object args)
        {
            Debug.WriteLine("Dismissed Thread ID: " + System.Environment.CurrentManagedThreadId);

            // uncomment this to see splash screen for longer
            //await ImageService.Current.DoLongRunningInitializationAsync();

            _uiContext.Post((s) =>
                {
                    Debug.WriteLine("Dispatched Thread ID: " + System.Environment.CurrentManagedThreadId);

                    Window.Current.Content = rootFrame;

                    if (_tileArguments.StartsWith("category="))
                    {
                        var categories = ImageService.Current.GetCategories();

                        string categoryText = _tileArguments.Substring(
                            _tileArguments.IndexOf('=') + 1);

                        Debug.WriteLine("Activated with category: " + categoryText);

                        var vm = new CategoryBrowserViewModel();

                        foreach (PhotoCategory category in categories)
                        {
                            vm.AllCategories.Add(category);

                            if (CategoryPinningService.Current.CategoryIsMatch(
                                category, categoryText))
                                vm.Category = category;
                        }

                        rootFrame.Navigate(typeof(CategoryBrowserPage), vm);
                    }
                    else
                    {
                        rootFrame.Navigate(typeof(MainPage));
                    }

                },
                null);
        }

        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e">Event data that describes how this page was reached.  The Parameter
        /// property is typically used to configure the page.</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
        }
    }
}
