﻿using PhotoBrowser.Model;
using System;
using Windows.UI.StartScreen;

namespace PhotoBrowser.Services
{
    public class CategoryPinningService
    {
        private static CategoryPinningService _current;
        public static CategoryPinningService Current
        {
            get
            {
                if (_current == null)
                    _current = new CategoryPinningService();

                return _current;
            }
        }

        public bool CategoryTileExists(PhotoCategory category)
        {
            if (category == null)
                return false;

            return SecondaryTile.Exists(
                CategoryTileIDFromCategory(category));
        }

        private string SanitizeCategory(PhotoCategory category)
        {
            return category.Category.Replace(" ", "");
        }

        private string CategoryTileIDFromCategory(PhotoCategory category)
        {
            string id = SanitizeCategory(category);

            id = "PhotoCategory." + id;
            id = id.Substring(0, Math.Min(id.Length, 64));

            return id;
        }

        public bool CategoryIsMatch(PhotoCategory category, string sanitizedCategory)
        {
            return (bool)(SanitizeCategory(category) == sanitizedCategory);
        }

        public SecondaryTile BuildCategoryPinTile(PhotoCategory category)
        {
            if (category != null && 
                !SecondaryTile.Exists(CategoryTileIDFromCategory(category)))
            {
                var logoUri = new Uri("ms-appx:///Assets/Logo.png");
                var wideLogoUri = new Uri("ms-appx:///Assets/WideLogo.png");

                var tileID = CategoryTileIDFromCategory(category);
                var displayName = "Photos for " + category.Category;
                var shortName = category.Category;
                var arguments = "category=" + SanitizeCategory(category);

                var secondaryTile = new SecondaryTile(
                    tileID, shortName, displayName, arguments,
                    TileOptions.ShowNameOnLogo | TileOptions.ShowNameOnWideLogo,
                    logoUri, wideLogoUri);

                secondaryTile.ForegroundText = ForegroundText.Light;

                return secondaryTile;
            }
            else
                return null;
        }
    }
}
