﻿using System;
using System.Collections.Generic;
using System.Linq;
using Windows.UI.Notifications;
using Windows.Data.Xml.Dom;
using PhotoBrowser.Model;

namespace PhotoBrowser.Services
{
    public class TileNotificationService
    {
        private static TileNotificationService _current;
        public static TileNotificationService Current
        {
            get
            {
                if (_current == null)
                    _current = new TileNotificationService();

                return _current;
            }
        }


        public void CreateSimpleSquareTileNotifications(IList<PhotoCategory> categories)
        {
            var tileXml = TileUpdateManager.GetTemplateContent(
                TileTemplateType.TileSquareBlock);

            int imageCount = ImageService.Current.GetPhotos().Count;

            var visualElement = tileXml.GetElementsByTagName("visual")[0];

            var attribute = tileXml.CreateAttribute("branding");
            attribute.NodeValue = "name";
            visualElement.Attributes.SetNamedItem(attribute);

            var textElements = tileXml.GetElementsByTagName("text");
            var countElement = (XmlElement)textElements[0];
            var textElement = (XmlElement)textElements[1];

            countElement.AppendChild(tileXml.CreateTextNode(imageCount.ToString()));
            textElement.AppendChild(tileXml.CreateTextNode("Photos"));

            var notification = new TileNotification(tileXml);

            
            var updater = TileUpdateManager.CreateTileUpdaterForApplication();
            updater.Update(notification);
        }


        private void SetBrandingToName(XmlDocument tileXml)
        {
            var visualElement = tileXml.GetElementsByTagName("visual")[0];

            var attribute = tileXml.CreateAttribute("branding");
            attribute.NodeValue = "name";
            visualElement.Attributes.SetNamedItem(attribute);
        }

        private TileNotification CreateTileWideImageCollectionNotification(
            IList<PhotoCategory> categories, string tag)
        {
            var tileXml = TileUpdateManager.GetTemplateContent(
                TileTemplateType.TileWideImageCollection);

            SetBrandingToName(tileXml);

            var imageElements = tileXml.GetElementsByTagName("image");

            var rnd = new Random();

            for (int i = 0; i < 5; i++)
            {
                var element = (XmlElement)imageElements[i];
                var srcAttribute = tileXml.CreateAttribute("src");

                int categoryIndex = i % categories.Count;
                int photoIndex =
                    rnd.Next(0, categories[categoryIndex].Photos.Count - 1);

                srcAttribute.NodeValue =
                    categories[categoryIndex]
                    .Photos[photoIndex].ThumbnailUri.ToString();

                element.Attributes.SetNamedItem(srcAttribute);
            }

            var notification = new TileNotification(tileXml);
            notification.Tag = tag;

            return notification;
        }

        private TileNotification CreateTileWideImageTextNotification(
            IList<PhotoCategory> categories, string tag)
        {
            var tileXml = TileUpdateManager.GetTemplateContent(
                TileTemplateType.TileWideImageAndText02);

            SetBrandingToName(tileXml);

            var imageElement = tileXml.GetElementsByTagName("image")[0];
            var categoryNameElement = tileXml.GetElementsByTagName("text")[0];
            var photoNameElement = tileXml.GetElementsByTagName("text")[1];

            var rnd = new Random();

            var category = categories[rnd.Next(0, categories.Count-1)];
            categoryNameElement.AppendChild(
                tileXml.CreateTextNode(category.Category));

            var photo = category.Photos[rnd.Next(0, category.Photos.Count-1)];
            photoNameElement.AppendChild(
                tileXml.CreateTextNode(photo.DisplayName));

            var srcAttribute = tileXml.CreateAttribute("src");

            srcAttribute.NodeValue = photo.ThumbnailUri.ToString();
            imageElement.Attributes.SetNamedItem(srcAttribute);

            var notification = new TileNotification(tileXml);
            notification.Tag = tag;

            return notification;
        }

        public void CreateWideTileNotifications(IList<PhotoCategory> categories)
        {
            var updater = TileUpdateManager.CreateTileUpdaterForApplication();
            updater.EnableNotificationQueue(true);

            for (int i = 0; i < 5; i++)
            {
                if (i % 2 > 0)
                {
                    updater.Update(CreateTileWideImageTextNotification(
                        categories, "tile.t." + i));
                }
                else
                {
                    updater.Update(CreateTileWideImageCollectionNotification(
                        categories, "tile.i." + i));
                }
            }
        }


        //public void CreateWideTileNotifications(IList<PhotoCategory> categories)
        //{
        //    var tileXml = TileUpdateManager.GetTemplateContent(
        //        TileTemplateType.TileWideImageCollection);

        //    var visualElement = tileXml.GetElementsByTagName("visual")[0];

        //    var attribute = tileXml.CreateAttribute("branding");
        //    attribute.NodeValue = "name";
        //    visualElement.Attributes.SetNamedItem(attribute);

        //    var imageElements = tileXml.GetElementsByTagName("image");

        //    var rnd = new Random();

        //    for (int i = 0; i < 5; i++)
        //    {
        //        var element = (XmlElement)imageElements[i];
        //        var srcAttribute = tileXml.CreateAttribute("src");

        //        int categoryIndex = i % categories.Count;
        //        int photoIndex =
        //            rnd.Next(0, categories[categoryIndex].Photos.Count - 1);

        //        srcAttribute.NodeValue =
        //            categories[categoryIndex]
        //            .Photos[photoIndex].ThumbnailUri.ToString();

        //        element.Attributes.SetNamedItem(srcAttribute);
        //    }


        //    var notification = new TileNotification(tileXml);

        //    var updater = TileUpdateManager.CreateTileUpdaterForApplication();
        //    updater.Update(notification);
        //}


    }
}
