﻿using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using PhotoBrowser.Model;
using PhotoBrowser.Services;
using System;
using System.Collections.ObjectModel;
using Windows.UI.StartScreen;

namespace PhotoBrowser.ViewModel
{
    public class CategoryBrowserViewModel : ViewModelBase
    {
        private ObservableCollection<PhotoCategory> _allCategories =
            new ObservableCollection<PhotoCategory>();

        public ObservableCollection<PhotoCategory> AllCategories
        {
            get { return _allCategories; }
            set
            {
                Set<ObservableCollection<PhotoCategory>>(
                    () => AllCategories,
                    ref _allCategories, 
                    value);
            }
        }

        private PhotoCategory _category;
        public PhotoCategory Category
        {
            get { return _category; }
            set
            {
                Set<PhotoCategory>(() => Category, ref _category, value);
                DeleteAllPhotosCommand.RaiseCanExecuteChanged();

                if (Category != null &&
                    Category.Photos != null &&
                    Category.Photos.Count > 0)
                    SelectedPhoto = Category.Photos[0];
            }
        }






        public CategoryBrowserViewModel()
        {
            DeleteAllPhotosCommand = new RelayCommand(
                () => DeleteAllPhotos(),
                () => CanDeleteAllPhotos());

            RotateSelectedPhotoCommand = new RelayCommand(
                () => RotateSelectedPhoto(),
                () => CanRotateSelectedPhoto());

            LikeSelectedPhotoCommand = new RelayCommand(
                () => LikeSelectedPhoto(),
                () => CanLikeSelectedPhoto());

            DislikeSelectedPhotoCommand = new RelayCommand(
                () => DislikeSelectedPhoto(),
                () => CanDislikeSelectedPhoto());

            PinCategoryCommand = new RelayCommand(
                () => PinCategory(),
                () => CanPinCategory());
        }


        private Photo _selectedPhoto;
        public Photo SelectedPhoto
        {
            get { return _selectedPhoto; }
            set
            {
                Set<Photo>(
                    () => SelectedPhoto, 
                    ref _selectedPhoto, value);

                RotateSelectedPhotoCommand.RaiseCanExecuteChanged();
                LikeSelectedPhotoCommand.RaiseCanExecuteChanged();
                DislikeSelectedPhotoCommand.RaiseCanExecuteChanged();
            }
        }


        public RelayCommand DeleteAllPhotosCommand
        {
            get;
            private set;
        }

        private void DeleteAllPhotos()
        {
            if (Category != null)
            {
                ToastNotificationService.Current.NotifyCategoryDeleted(Category);

                Category.Photos.Clear();

                ImageService.Current.RemoveImagesWithCategory(Category.Category);

                AllCategories.Remove(Category);

                Category = null;
                SelectedPhoto = null;
            }
        }

        public bool CanDeleteAllPhotos()
        {
            return Category != null &&
                Category.Photos != null &&
                Category.Photos.Count > 0;
        }



        public RelayCommand RotateSelectedPhotoCommand
        {
            get;
            private set;
        }

        private void RotateSelectedPhoto()
        {
            if (SelectedPhoto != null)
            {
                SelectedPhoto.RotationAngle = 
                    (SelectedPhoto.RotationAngle + 90) % 360;
            }
        }

        public bool CanRotateSelectedPhoto()
        {
            return SelectedPhoto != null;
        }



        public RelayCommand LikeSelectedPhotoCommand
        {
            get;
            private set;
        }

        private void LikeSelectedPhoto()
        {
            if (SelectedPhoto != null)
            {
                SelectedPhoto.LikesCount += 1;
            }
        }

        public bool CanLikeSelectedPhoto()
        {
            return SelectedPhoto != null;
        }


        public RelayCommand DislikeSelectedPhotoCommand
        {
            get;
            private set;
        }

        private void DislikeSelectedPhoto()
        {
            if (SelectedPhoto != null)
            {
                SelectedPhoto.DislikesCount += 1;
            }
        }

        public bool CanDislikeSelectedPhoto()
        {
            return SelectedPhoto != null;
        }







        public RelayCommand PinCategoryCommand
        {
            get;
            private set;
        }

        public async void PinCategory()
        {
            var tile = CategoryPinningService.Current.BuildCategoryPinTile(Category);

            await tile.RequestCreateAsync();
        }

        public bool CanPinCategory()
        {
            return !CategoryPinningService.Current.CategoryTileExists(Category);
        }
    }
}
