﻿using PhotoBrowser.Model;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Windows.Storage;
using Windows.Storage.Streams;

namespace PhotoBrowser.Services
{
    public class ImageService
    {
        private static ImageService _current;
        public static ImageService Current
        {
            get
            {
                if (_current == null)
                    _current = new ImageService();

                return _current;
            }
        }

        private List<Photo> _photos;

        public IList<Photo> GetPhotos()
        {
            if (_photos == null)
            {
                string[] fileNames = new string[]
                {
                    "Autumn Trees 2007.jpg", "Nature",
                    "Beetle on Azalea.jpg", "Nature",
                    "Blue Marbles Render.jpg", "Computer Graphics", 
                    "Christmas Silver Blue Glitter Poinsettias.png", "Christmas", 
                    "Christmas Tree Butterfly Fruit.jpg", "Christmas", 
                    "Christmas Under the Sea Tree Purple.jpg", "Christmas", 
                    "Cucumber Beetle on Pumpkin.jpg", "Nature", 
                    "Fall Crisp Blue Lake.png", "Nature", 
                    "MFOS Synth Board 2.png", "Electronics", 
                    "MMID 302 Union Bridge Dec 15, 2004.png", "Trains", 
                    "Nebula.jpg", "Computer Graphics", 
                    "Plasma Chamber.jpg", "Computer Graphics", 
                    "Shadowed Pumpkin.jpg", "Nature", 
                    "Synth Background.jpg", "Electronics", 
                    "Synth Panel.jpg", "Electronics"
                };

                _photos = new List<Photo>();

                for (int i = 0; i < fileNames.Length; i += 2)
                {
                    string name = fileNames[i];
                    string category = fileNames[i + 1];

                    string displayName = name.Substring(0, name.LastIndexOf('.'));
                    string uriRoot = "ms-appx:///Pictures/"; // IMPORTANT CHANGE FROM PREVIOUS SOURCE

                    _photos.Add(
                        new Photo()
                            {
                                ImageUri = new Uri(uriRoot + name),
                                DisplayName = displayName,
                                ThumbnailUri = 
                                    new Uri(uriRoot + "thumbnail_" + displayName + ".jpg"),
                                Category = category,
                                LikesCount=9001

                            }
                        );
                }
            }

            return _photos;
        }

        public void RemoveImagesWithCategory(string category)
        {
            if (_photos != null)
            {
                _photos.RemoveAll(s => s.Category == category);
            }
        }

        // simulates some long-running code
        public Task<int> DoLongRunningInitializationAsync()
        {
            return Task.Delay(5000).ContinueWith((t) => { return 42; });
        }


        public IList<PhotoCategory> GetCategories()
        {
            var photos = GetPhotos();

            return photos.GroupBy(p => p.Category)
                .Select(p => new PhotoCategory()
                {
                    Category = p.Key,
                    Photos = p.ToList()
                }).ToList();
        }


        public async Task<RandomAccessStreamReference> GetFileStreamFromPhotoAsync(Photo photo)
        {
            var f = await StorageFile.GetFileFromApplicationUriAsync(photo.ImageUri);

            return RandomAccessStreamReference.CreateFromFile(f);
        }

        public async Task<RandomAccessStreamReference> GetThumbnailStreamFromPhotoAsync(Photo photo)
        {
            var f = await StorageFile.GetFileFromApplicationUriAsync(photo.ThumbnailUri);

            return RandomAccessStreamReference.CreateFromFile(f);
        }


        //public Task<IReadOnlyList<StorageFile>> GetImagesAsync()
        //{
        //    return KnownFolders.PicturesLibrary.GetFilesAsync()
        //        .AsTask<IReadOnlyList<StorageFile>>();
        //}


        //public object GetVirtualizedImagesVector()
        //{
        //    var queryOptions = new QueryOptions();
        //    queryOptions.FolderDepth = FolderDepth.Deep;
        //    queryOptions.IndexerOption = IndexerOption.UseIndexerWhenAvailable;
        //    queryOptions.SortOrder.Clear();

        //    var sortEntry = new SortEntry();
        //    sortEntry.PropertyName = "System.FileName";
        //    sortEntry.AscendingOrder = true;

        //    queryOptions.SortOrder.Add(sortEntry);

        //    var fileQuery = KnownFolders.PicturesLibrary
        //        .CreateFileQueryWithOptions(queryOptions);

        //    const uint thumbnailSize = 250;

        //    var fileInformationFactory = new FileInformationFactory(
        //        fileQuery, 
        //        ThumbnailMode.PicturesView,
        //        thumbnailSize, 
        //        ThumbnailOptions.UseCurrentScale, true);
            
        //    var files = fileInformationFactory.GetVirtualizedFilesVector();

        //    return files;
        //} 
    }
}
