﻿using GalaSoft.MvvmLight;
using PhotoBrowser.Model;
using PhotoBrowser.Services;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.UI.Xaml.Data;

namespace PhotoBrowser.ViewModel
{
    public class SearchViewModel : ViewModelBase
    {
        public SearchViewModel()
        {
        }

        private IEnumerable<Photo> _searchResults;
        public IEnumerable<Photo> SearchResults
        {
            get { return _searchResults; }
            private set 
            { 
                Set<IEnumerable<Photo>>(() => SearchResults, 
                            ref _searchResults, value); 
            }
        }

        private string _queryText;
        public string QueryText
        {
            get { return _queryText; }
            set 
            { 
                Set<string>(() => QueryText, ref _queryText, value);
                UpdateSearchResults();
            }
        }

        // filter based on query text
        private void UpdateSearchResults()
        {
            var photos = ImageService.Current.GetPhotos();


            if (string.IsNullOrEmpty(QueryText))
                SearchResults = photos;
            else
            {
                var q = QueryText.ToLower();

                var results = from Photo p in photos
                              where (p.DisplayName.ToLower().Contains(q) ||
                                    p.Category.ToLower().Contains(q))
                              select p;

                SearchResults = results;
            }
        }


        public CategoryBrowserViewModel ConstructCategoryBrowserViewModel(Photo photo)
        {
            var vm = new CategoryBrowserViewModel();

            foreach (PhotoCategory c in ImageService.Current.GetCategories())
                vm.AllCategories.Add(c);

            vm.Category = vm.AllCategories
                .FirstOrDefault<PhotoCategory>(c => c.Category == photo.Category);

            vm.SelectedPhoto = photo;

            return vm;
        }

    }
}
