﻿using System;
using Windows.ApplicationModel.Activation;
using Windows.Storage;
using Windows.Storage.Pickers.Provider;
using Windows.Storage.Search;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;

namespace DemoApp
{
    public sealed partial class FileOpenPickerPage : Page
    {
        FileOpenPickerActivatedEventArgs _args;

        public FileOpenPickerPage(FileOpenPickerActivatedEventArgs args)
        {
            this.InitializeComponent();

            _args = args;

            LoadFiles();

            if (_args.FileOpenPickerUI.SelectionMode == FileSelectionMode.Multiple)
                FileList.SelectionMode = ListViewSelectionMode.Multiple;
            else
                FileList.SelectionMode = ListViewSelectionMode.Single;

            _args.FileOpenPickerUI.Title = "Pete's cool files";

            FileList.SelectionChanged += FileList_SelectionChanged;
            _args.FileOpenPickerUI.FileRemoved += FileOpenPickerUI_FileRemoved;
        }

        void FileOpenPickerUI_FileRemoved(FileOpenPickerUI sender, FileRemovedEventArgs args)
        {
            if (FileList.SelectionMode == ListViewSelectionMode.Multiple)
            {
                foreach (StorageFile file in FileList.SelectedItems)
                {
                    if (file.Path == args.Id)
                    {
                        FileList.SelectedItems.Remove(file);
                        break;
                    }
                }
            }
            else
            {
                if (FileList.SelectedItem != null)
                {
                    if (((StorageFile)(FileList.SelectedItem)).Path ==
                        args.Id)
                        FileList.SelectedItem = null;
                }
                    
            }
        }

        void FileList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            foreach (StorageFile file in e.AddedItems)
                _args.FileOpenPickerUI.AddFile(file.Path, file);

            foreach (StorageFile file in e.RemovedItems)
                _args.FileOpenPickerUI.RemoveFile(file.Path);
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {          
        }


        private async void LoadFiles()
        {
            var options = new QueryOptions(CommonFileQuery.OrderByDate,
                new string[] { ".jpg", ".png" });

            var result = KnownFolders.PicturesLibrary.CreateFileQueryWithOptions(
                options);

            var files = await result.GetFilesAsync();

            FileList.ItemsSource = files;
        }        
    }
}
