﻿using System;
using DemoApp.ViewModel;
using Windows.Storage;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;
using Windows.UI.Popups;
using Windows.Storage.Pickers;

namespace DemoApp
{
    public sealed partial class MainPage : Page
    {
        private MainViewModel _vm = new MainViewModel();

        public MainPage()
        {
            this.InitializeComponent();

            DataContext = _vm;
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
        }

        private void GetDocumentLibraryClick(object sender, RoutedEventArgs e)
        {
            _vm.LoadDocumentsLibrary();
        }

        private void GetMusicLibraryClick(object sender, RoutedEventArgs e)
        {
            _vm.LoadMusicLibrary();
        }

        private void GetRemovableDevicesClick(object sender, RoutedEventArgs e)
        {
            _vm.LoadRemovableDevices();
        }


        private async void OpenFilePickerClick(object sender, RoutedEventArgs e)
        {
            var picker = new FileOpenPicker();

            picker.ViewMode = PickerViewMode.Thumbnail;
            picker.SuggestedStartLocation = PickerLocationId.DocumentsLibrary;
            picker.FileTypeFilter.Add(".jpg");
            picker.FileTypeFilter.Add(".png");
            picker.FileTypeFilter.Add(".satxt");
            picker.FileTypeFilter.Add(".txt");
            picker.FileTypeFilter.Add(".docx");

            var file = await picker.PickSingleFileAsync();

            if (file != null)
            {
                // file picked. You can now work with it
                var dlg = new MessageDialog(file.DisplayName + " was picked.");
                await dlg.ShowAsync();
            }
            else
            {
                var dlg = new MessageDialog("No file picked.");
                await dlg.ShowAsync();
            }
        }

        
        private async void CreateFileClick(object sender, RoutedEventArgs e)
        {
            string folderName = FolderName.Text.Trim();
            string fileName = FileName.Text.Trim();
            string contents = Contents.Text.Trim();

            var file = await _vm.CreateFile(folderName, fileName);

            if (file != null)
            {
                await FileIO.AppendTextAsync(file, contents);
            }
            else
            {
                var dlg = new MessageDialog("Unable to create file.");
                await dlg.ShowAsync();
            }
        }

    }
}
