﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.Storage;
using Windows.Storage.Search;

namespace DemoApp.ViewModel
{
    public class MainViewModel: INotifyPropertyChanged
    {

        private ObservableCollection<IStorageItem> _childItems =
            new ObservableCollection<IStorageItem>();
        public ObservableCollection<IStorageItem> ChildItems
        {
            get { return _childItems; }
        }

        private StorageFolder _currentFolder;
        public StorageFolder CurrentFolder
        {
            get { return _currentFolder; }
            set 
            { 
                _currentFolder = value;
                NotifyPropertyChanged("CurrentFolder");
                //UpdateChildren();
                UpdateChildrenWithQuery();
            }
        }


        private async void UpdateChildrenWithQuery()
        {
            var options = new QueryOptions(CommonFileQuery.OrderByDate,
                new string[] { ".satxt", ".docx", ".txt" });

            var result = KnownFolders.DocumentsLibrary.CreateItemQueryWithOptions(
                options);

            var items = await result.GetItemsAsync();

            ChildItems.Clear();

            foreach (var item in items)
            {
                ChildItems.Add(item);
            }  
        }


        private async void UpdateChildren()
        {
            var items = await CurrentFolder.GetItemsAsync();

            ChildItems.Clear();

            foreach (var item in items)
            {
                ChildItems.Add(item);
            }
        }

        public void LoadDocumentsLibrary()
        {
            CurrentFolder = KnownFolders.DocumentsLibrary;
        }

        public void LoadMusicLibrary()
        {
            CurrentFolder = KnownFolders.MusicLibrary;
        }

        public void LoadRemovableDevices()
        {
            CurrentFolder = KnownFolders.RemovableDevices;
        }




        public async Task<StorageFile> CreateFile(
                            string subFolderName, string fileName)
        {
            if (CurrentFolder != null)
            {
                var folder = await CurrentFolder.CreateFolderAsync(
                    subFolderName,
                    CreationCollisionOption.GenerateUniqueName);

                return await folder.CreateFileAsync(
                    fileName,
                    CreationCollisionOption.GenerateUniqueName)
                    .AsTask();
            }
            else
            {
                return null;
            }
        }








        public event PropertyChangedEventHandler PropertyChanged;
        protected void NotifyPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, 
                    new PropertyChangedEventArgs(propertyName));
        }

    }
}
