﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.Threading;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Windows.Web.Syndication;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace AsyncExamples
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();
        }

        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e">Event data that describes how this page was reached.  The Parameter
        /// property is typically used to configure the page.</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            //LoadFeedUsingAwait();
            //LoadFeedUsingLongForm();
            //LoadFeedUsingLongFormWithCancel();

            //LoadHttpWithAwait();
            //LoadHttpUsingLongForm();
            LoadHttpUsingLongFormWithCancel();
        }

        // WinRT Async pattern
        private async void LoadFeedUsingAwait()
        {
            var client = new SyndicationClient();

            var uri = new Uri("http://feeds.feedburner.com/PeteBrown");
            var feed = await client.RetrieveFeedAsync(uri);

            Debug.WriteLine(feed.Title.Text);
        }

        // .NET TPL Async pattern
        private async void LoadHttpWithAwait()
        {
            var client = new HttpClient();

            var uri = new Uri("http://10rem.net");
            var feed = await client.GetAsync(uri);

            var results = await feed.Content.ReadAsStringAsync();

            Debug.WriteLine(results);
        }



        // WinRT Async pattern
        private void LoadFeedUsingLongForm()
        {
            var client = new SyndicationClient();

            var uri = new Uri("http://feeds.feedburner.com/PeteBrown");
            //var uri = new Uri("http://10rem.net/blog/rss.aspx");
            var op = client.RetrieveFeedAsync(uri);

            op.Completed = (info, status) =>
            {
                switch (status)
                {
                    case AsyncStatus.Canceled:
                        Debug.WriteLine("Operation was canceled");
                        break;
                    case AsyncStatus.Completed:
                        Debug.WriteLine("Operation was completed");
                        Debug.WriteLine(info.GetResults().Title.Text);
                        break;
                    case AsyncStatus.Error:
                        Debug.WriteLine("Operation had an error:");
                        Debug.WriteLine(info.ErrorCode.Message);
                        break;
                }
            };

            op.Progress = (info, progress) =>
            {
                Debug.WriteLine(string.Format("{0}/{1} bytes {2:P}",
                    progress.BytesRetrieved,
                    progress.TotalBytesToRetrieve,
                    (float)progress.BytesRetrieved / (float)progress.TotalBytesToRetrieve));
            };
        }


        // .NET TPL Async pattern
        private void LoadHttpUsingLongForm()
        {
            var client = new HttpClient();

            var uri = new Uri("http://10rem.net");

            client.GetAsync(uri).ContinueWith((t) =>
            {
                t.Result.Content.ReadAsStringAsync().ContinueWith((t2) =>
                {
                    Debug.WriteLine(t2.Result);
                });

            });
        }

        // .NET TPL Async pattern
        private void LoadHttpUsingLongForm2()
        {
            var client = new HttpClient();

            var uri = new Uri("http://10rem.net");

            client.GetAsync(uri).ContinueWith((t) =>
                t.Result.Content.ReadAsStringAsync().ContinueWith((t2) =>
                    Debug.WriteLine(t2.Result)));
        }




        // WinRT Async pattern
        private void LoadFeedUsingLongFormWithCancel()
        {
            var client = new SyndicationClient();

            //var uri = new Uri("http://feeds.feedburner.com/PeteBrown");
            var uri = new Uri("http://10rem.net/blog/rss.aspx");

            var op = client.RetrieveFeedAsync(uri);

            op.Completed = (info, status) =>
            {
                switch (status)
                {
                    case AsyncStatus.Canceled:
                        Debug.WriteLine("Operation was canceled");
                        break;
                    case AsyncStatus.Completed:
                        Debug.WriteLine("Operation was completed");
                        Debug.WriteLine(info.GetResults().Title.Text);
                        break;
                    case AsyncStatus.Error:
                        Debug.WriteLine("Operation had an error:");
                        Debug.WriteLine(info.ErrorCode.Message);
                        break;
                }
            };

            bool alreadyCancelled = false;
            op.Progress = (info, progress) =>
            {
                Debug.WriteLine(string.Format("{0}/{1} bytes {2:P}",
                    progress.BytesRetrieved,
                    progress.TotalBytesToRetrieve,
                    (float)progress.BytesRetrieved / (float)progress.TotalBytesToRetrieve));

                if (!alreadyCancelled)
                {
                    if (progress.BytesRetrieved > 5000)
                    {
                        info.Cancel();
                        alreadyCancelled = true;
                    }
                }
            };
        }


        // .NET TPL Async pattern
        private void LoadHttpUsingLongFormWithCancel()
        {
            var client = new HttpClient();

            var uri = new Uri("http://10rem.net");

            var cancelTokenSource = new CancellationTokenSource();
            var cancelToken = cancelTokenSource.Token;

            client.GetAsync(uri, cancelToken).ContinueWith((t) =>
            {
                if (t.IsCanceled)
                    Debug.WriteLine("Download was cancelled");
                else
                    t.Result.Content.ReadAsStringAsync().ContinueWith((t2) =>
                        Debug.WriteLine(t2.Result));
            });

            cancelTokenSource.CancelAfter(5);
        }




        private void ConvertWinRTToTask()
        {
            var client = new SyndicationClient();

            var uri = new Uri("http://feeds.feedburner.com/PeteBrown");

            var op = client.RetrieveFeedAsync(uri).AsTask<SyndicationFeed, RetrievalProgress>();

            op.ContinueWith((t) =>
            {
                if (t.IsCanceled)
                    Debug.WriteLine("Feed retrieval was cancelled");
                else
                    Debug.WriteLine(t.Result.Title);
            });

        }


        private void ConvertTaskToWinRT()
        {
            var client = new HttpClient();

            var uri = new Uri("http://10rem.net");

            var op = client.GetAsync(uri).AsAsyncOperation<HttpResponseMessage>();

            op.Completed = (info, status) =>
            {
                switch (status)
                {
                    case AsyncStatus.Canceled:
                        Debug.WriteLine("Operation was canceled");
                        break;
                    case AsyncStatus.Completed:
                        Debug.WriteLine("Operation was completed");
                        break;
                    case AsyncStatus.Error:
                        Debug.WriteLine("Operation had an error:");
                        Debug.WriteLine(info.ErrorCode.Message);
                        break;
                }
            };

        }
    }
}
