﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Net;
using System.Net.Http;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Xml.Serialization;
using NetworkingClient.Model;
using Windows.System.UserProfile;

namespace NetworkingClient.ViewModels
{
    class MainViewModel : INotifyPropertyChanged
    {
        private ObservableCollection<InstantMessage> _messages = 
            new ObservableCollection<InstantMessage>();
        
        public ObservableCollection<InstantMessage> Messages        
        {
            get { return _messages; }
            set
            {
                if (_messages != value)
                {
                    _messages = value;
                    NotifyPropertyChanged("Messages");
                }
            }
        }


        //public async void LoadMessages()
        //{
        //    var client = new HttpClient();
        //    var uri = new Uri("http://localhost:4621/api/messages");

        //    var response = await client.GetAsync(uri);
            
        //    var data = await response.Content.ReadAsStringAsync();

        //    Debug.WriteLine(data);
        //}


        public async void LoadMessages()
        {
            var uri = new Uri("http://localhost:4621/api/messages");
            var request = WebRequest.CreateHttp(uri);

            request.Method = "GET";
            request.Accept = "text/xml";

            var response = await request.GetResponseAsync();
            var stream = response.GetResponseStream();

            var deserializer = new XmlSerializer(typeof(InstantMessage[]), 
                "http://schemas.datacontract.org/2004/07/NetworkingClient.Model");

            var messages = (InstantMessage[])deserializer.Deserialize(stream);

            _messages.Clear();

            foreach (InstantMessage msg in messages)
                _messages.Add(msg);
        }


        //public async void LoadMessages()
        //{
        //    var uri = new Uri("http://localhost:4621/api/messages");
        //    var request = WebRequest.CreateHttp(uri);

        //    request.Method = "GET";
        //    request.Accept = "application/json";

        //    var response = await request.GetResponseAsync();
        //    var stream = response.GetResponseStream();

        //    var settings = new DataContractJsonSerializerSettings();
        //    string dateFormat = "yyyy-MM-ddTHH:mm:ss";
        //    settings.DateTimeFormat = new DateTimeFormat(dateFormat);

        //    var deserializer = new DataContractJsonSerializer(typeof(InstantMessage[]), settings);

        //    var messages = (InstantMessage[])deserializer.ReadObject(stream);

        //    _messages.Clear();
        //    foreach (InstantMessage msg in messages)
        //        _messages.Add(msg);
        //}

        private InstantMessage _selectedMessage;
        public InstantMessage SelectedMessage
        {
            get { return _selectedMessage; }
            set { _selectedMessage = value; NotifyPropertyChanged("SelectedMessage"); }
        }

        private const string RootMessagesServiceUrl = 
            "http://localhost:4621/api/messages/";

        private async void SendRequest(string verb, InstantMessage message, string url)
        {
            var uri = new Uri(url);
            var request = WebRequest.CreateHttp(uri);

            request.Method = verb;
            request.ContentType = "application/json";

            var requestStream = await request.GetRequestStreamAsync();

            var settings = new DataContractJsonSerializerSettings();
            string dateFormat = "yyyy-MM-ddTHH:mm:ss";
            settings.DateTimeFormat = new DateTimeFormat(dateFormat);

            var serializer = new DataContractJsonSerializer(typeof(InstantMessage), settings);
            serializer.WriteObject(requestStream, message);

            var response = await request.GetResponseAsync();
        }

        public void UpdateSelectedMessage()
        {
            if (SelectedMessage != null)
            {
                SelectedMessage.Text += " upd";

                SendRequest("PUT", SelectedMessage, RootMessagesServiceUrl + SelectedMessage.Id);
            }
        }

        public async void AddMessage()
        {
            var message = new InstantMessage();

            message.Id = (int)(DateTime.Now.Ticks & 0xFFFF);
            message.Timestamp = DateTime.Now;
            message.From = await UserInformation.GetDisplayNameAsync();
            message.To = "Pete";
            message.Text = "Some random text from viewmodel";

            SendRequest("POST", message, RootMessagesServiceUrl);
        }

        public async void DeleteSelectedMessage()
        {
            if (SelectedMessage != null)
            {
                var uri = new Uri(RootMessagesServiceUrl + SelectedMessage.Id);
                var request = WebRequest.CreateHttp(uri);

                request.Method = "DELETE";

                var response = await request.GetResponseAsync();
            }
        }


        public event PropertyChangedEventHandler PropertyChanged;

        protected void NotifyPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
