﻿using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using GalaSoft.MvvmLight.Threading;
using SocketApp.Model;
using SocketApp.Services;
using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Threading.Tasks;
using Windows.Networking;
using Windows.Networking.Connectivity;
using Windows.Networking.Sockets;
using Windows.Storage.Streams;
using Windows.System.UserProfile;

namespace SocketApp.ViewModel
{
    public class MainViewModel : ViewModelBase
    {
        private IMessageService _messageService;

        public MainViewModel()
        {
            //_messageService = new TcpStreamMessageService();
            _messageService = new UdpMessageService();

            _messageService.ChatMessageReceived += OnServiceChatMessageReceived;
            _messageService.ConnectionReceived += OnServiceConnectionReceived;
            _messageService.PlayerExited += OnServicePlayerExited;
            _messageService.PlayerJoined += OnServicePlayerJoined;

            ChatMessages = new ObservableCollection<ChatMessage>();
            ConnectionStatus = "Not connected.";
            ServerAddress = "pete-surface64";

            PostNewMessageCommand = new RelayCommand(
                () => PostNewMessage(),
                () => CanPostNewMessage());

            ListenCommand = new RelayCommand(
                () => Listen(),
                () => CanListen());

            ConnectCommand = new RelayCommand(
                () => Connect(),
                () => CanConnect());

            InitializePlayer();
            CreateNewMessage();
        }


        private Player _player;
        public Player Player
        {
            get { return _player; }
            set { Set<Player>(() => Player, ref _player, value); }
        }

        private async void InitializePlayer()
        {
            _player = new Player();
            _player.Name = await UserInformation.GetDisplayNameAsync();
        }


        public ObservableCollection<ChatMessage> ChatMessages { get; set; }

        private ChatMessage _newMessage;
        public ChatMessage NewMessage
        {
            get { return _newMessage; }
            set { Set<ChatMessage>(() => NewMessage, ref _newMessage, value); }
        }

        public RelayCommand PostNewMessageCommand
        {
            get; private set;
        }

        private void CreateNewMessage()
        {
            if (NewMessage != null)
                NewMessage.PropertyChanged -= NewMessage_PropertyChanged;

            NewMessage = new ChatMessage();
            NewMessage.Player = Player;
            NewMessage.PropertyChanged += NewMessage_PropertyChanged;
        }

        void NewMessage_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Message")
            {
                PostNewMessageCommand.RaiseCanExecuteChanged();
            }
        }


        public async void PostNewMessage()
        {
            ChatMessages.Add(NewMessage);
            _messageService.SendChatMessage(NewMessage);

            CreateNewMessage();
        }

        public bool CanPostNewMessage()
        {
            //return NewMessage != null &&
            //    NewMessage.Message != null &&
            //    NewMessage.Message.Trim() != string.Empty;

            return true;
        }


        // Communications support
        private string _serverAddress;
        public string ServerAddress
        {
            get { return _serverAddress; }
            set { Set<string>(() => ServerAddress, ref _serverAddress, value); }
        }

        private string _connectionStatus;
        public string ConnectionStatus
        {
            get { return _connectionStatus; }
            set { Set<string>(() => ConnectionStatus, ref _connectionStatus, value); }
        }


        public RelayCommand ConnectCommand
        {
            get; private set;
        }
       
        public void Connect()
        {
            _messageService.Connect(Player, ServerAddress);

            ConnectionStatus = "Connected to server at " + ServerAddress + ".";
        }

        public bool CanConnect()
        {
            return true;
        }




        public RelayCommand ListenCommand
        {
            get; private set;
        }

        public void Listen()
        {
            _messageService.Listen(Player);

            ConnectionStatus = "Waiting for connection on: ";

            int i = 0;

            foreach (HostName name in _messageService.GetHostNames())
            {
                if (i > 0)
                    ConnectionStatus += " and ";

                ConnectionStatus += name.DisplayName;
                i++;
            }
        }

        public bool CanListen()
        {
            return true;
        }


        void OnServicePlayerJoined(object sender, PlayerJoinedEventArgs e)
        {
            DispatcherHelper.CheckBeginInvokeOnUI(() =>
            {
                ConnectionStatus = "Just joined: " +
                    e.Player.Name;
            });
        }

        void OnServicePlayerExited(object sender, PlayerExitedEventArgs e)
        {
            DispatcherHelper.CheckBeginInvokeOnUI(() =>
            {
                ConnectionStatus = "Just exited: " +
                    e.Player.Name;
            });
        }

        void OnServiceConnectionReceived(object sender, ConnectionReceivedEventArgs e)
        {
            DispatcherHelper.CheckBeginInvokeOnUI(() =>
            {
                ConnectionStatus = "Connection received from " +
                    e.HostName.DisplayName;
            });
        }

        void OnServiceChatMessageReceived(object sender, ChatMessageReceivedEventArgs e)
        {
            DispatcherHelper.CheckBeginInvokeOnUI(() =>
            {
                ChatMessages.Add(e.Message);
            });
        }
    }
}