﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SocketApp.Services
{
    public enum MessageId
    {
        StartMoveForward,
        StartMoveBackward,
        StopMove,
        StartRotateCounterClockwise,
        StartRotateClockwise,
        StopRotate
    }

    public class MessageReceivedEventArgs : EventArgs
    {
        public MessageId MessageId { get; set; }
        public double XParam { get; set; }
        public double YParam { get; set; }

        public MessageReceivedEventArgs(MessageId messageId)
        {
            MessageId = messageId;
            XParam = double.NaN;
            YParam = double.NaN;
        }

        public MessageReceivedEventArgs(MessageId messageId, double X)
        {
            MessageId = messageId;
            XParam = X;
            YParam = double.NaN;
        }

        public MessageReceivedEventArgs(MessageId messageId, double X, double Y)
        {
            MessageId = messageId;
            XParam = X;
            YParam = Y;
        }
    }

    public interface IMessageService
    {
        //void SendMessage(string message);

        void Connect(string address);
        void Disconnect();
        void Listen();

        void SendStartRotateCounterClockwise();
        void SendStartRotateClockwise();
        void SendStopTurning(double finalAngle);

        void SendStartMoveForward();
        void SendStartMoveBackward();
        void SendStopMoving(double finalX, double finalY);

        event EventHandler<MessageReceivedEventArgs> MessageReceived;

    }
}
