﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.Networking;
using Windows.Networking.Connectivity;
using Windows.Networking.Sockets;
using Windows.Storage.Streams;

namespace SocketApp.Services
{
    public class TcpStreamingSocketService  : IMessageService
    {
        private StreamSocket _socket;
        private StreamSocketListener _listener;

        //public const string PortOrService = "5150";
        public const string PortOrService = "80";
        
        public event EventHandler<MessageReceivedEventArgs> MessageReceived;
        public event EventHandler<ConnectionReceivedEventArgs> ConnectionReceived;
        
        // Connect to another machine that is already listening
        public async void Connect(string address)
        {
            var hostName = new HostName(address);

            _socket = new StreamSocket();
            await _socket.ConnectAsync(hostName, PortOrService);

            SendSimpleText("Hello!");
        }

        public void Disconnect()
        {
            if (_socket != null)
            {
                // todo: send "bye" message

                _socket.Dispose();
                _socket = null;
            }

            if (_listener != null)
            {
                _listener.Dispose();
                _listener = null;
            }
        }

        // listen for other connections
        public async void Listen()
        {
            _listener = new StreamSocketListener();
            _listener.ConnectionReceived += OnConnectionReceived;

            await _listener.BindServiceNameAsync(PortOrService);

            
        }

        //private Task _backgroundPollingTask;

        private void OnConnectionReceived(StreamSocketListener sender, StreamSocketListenerConnectionReceivedEventArgs args)
        {
            // set the socket to the one we got from listening
            _socket = args.Socket;

            Debug.WriteLine("Connection received from " +
                _socket.Information.RemoteHostName.DisplayName + ":" +
                _socket.Information.RemotePort);

            if (ConnectionReceived != null)
                ConnectionReceived(this, new ConnectionReceivedEventArgs() 
                    { RemoteHost = _socket.Information.RemoteHostName.DisplayName });

        }


        private async void InternalSendMessage(MessageType id, double finalX, double finalY, double angle, byte[] remainingMessage)
        {
            // format the start of the message and add remaining bytes

            var stream = _socket.OutputStream;

            var writer = new DataWriter(stream);

            writer.WriteInt32((int)id);
            writer.WriteDouble(finalX);
            writer.WriteDouble(finalY);
            writer.WriteDouble(angle);

            if (remainingMessage != null && remainingMessage.Length > 0)
                writer.WriteBytes(remainingMessage);

            await writer.StoreAsync();
        }


        public async void SendSimpleText(string text)
        {
            var stream = _socket.OutputStream;

            var writer = new DataWriter(stream);

            writer.WriteString(text);

            await writer.StoreAsync();
        }


        
        public void SendTextMessage(double finalX, double finalY, double angle, string message)
        {

        }

        public void SendEnlistment(double finalX, double finalY, double angle, string name, byte[] image)
        {
        }



        public void SendStartRotateCounterClockwise(double finalX, double finalY, double angle)
        {
            
        }

        public void SendStartRotateClockwise(double finalX, double finalY, double angle)
        {
            
        }

        public void SendStopRotating(double finalX, double finalY, double angle)
        {
            
        }



        public void SendStartMoveForward(double finalX, double finalY, double angle)
        {
            
        }

        public void SendStartMoveBackward(double finalX, double finalY, double angle)
        {
            
        }

        public void SendStopMoving(double finalX, double finalY, double angle)
        {
            
        }


    }
}
