﻿using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using GalaSoft.MvvmLight.Threading;
using SocketApp.Model;
using SocketApp.Services;
using System.Collections.ObjectModel;
using System.ComponentModel;
using Windows.Networking;
using Windows.Networking.Connectivity;
using Windows.UI.Core;

namespace SocketApp.ViewModel
{
    public class MainViewModel : ViewModelBase
    {
        private IMessageService _messageService;

        

        public MainViewModel()
        {
            Players = new ObservableCollection<Player>();
            ChatMessages = new ObservableCollection<ChatMessage>();
            ConnectionStatus = "Not connected.";
            //ServerAddress = "pete-surface64";
            ServerAddress = "192.168.1.8";

            _messageService = new TcpStreamingSocketService();

            _messageService.MessageReceived += OnMessageReceived;
            _messageService.ConnectionReceived += OnConnectionReceived;

            PostNewMessageCommand = new RelayCommand(
                () => PostNewMessage(),
                () => CanPostNewMessage());

            ListenCommand = new RelayCommand(
                () => Listen(),
                () => CanListen());

            ConnectCommand = new RelayCommand(
                () => Connect(),
                () => CanConnect());
        
        }


        // General
        private Player _me;
        public Player Me
        {
            get { return _me; }
            set { Set<Player>(() => Me, ref _me, value); }
        }

        public ObservableCollection<Player> Players { get; set; }



        // Chat support
        public ObservableCollection<ChatMessage> ChatMessages { get; set; }

        private ChatMessage _newMessage;
        public ChatMessage NewMessage
        {
            get { return _newMessage; }
            set { Set<ChatMessage>(() => NewMessage, ref _newMessage, value); }
        }

        public RelayCommand PostNewMessageCommand
        {
            get;
            private set;
        }

        private void CreateNewMessage()
        {
            if (NewMessage != null)
                NewMessage.PropertyChanged -= NewMessage_PropertyChanged;

            NewMessage = new ChatMessage();
            NewMessage.PropertyChanged += NewMessage_PropertyChanged;
        }

        void NewMessage_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Message")
            {
                PostNewMessageCommand.RaiseCanExecuteChanged();
            }
        }

        public void PostNewMessage()
        {
            NewMessage.From = Me;

            //ChatMessages.Add(NewMessage);
            // TODO: Post message to network

            CreateNewMessage();
        }

        public bool CanPostNewMessage()
        {
            return NewMessage != null &&
                NewMessage.Message != null &&
                NewMessage.Message.Trim() != string.Empty;
        }


        // Communications support
        private string _serverAddress;
        public string ServerAddress
        {
            get { return _serverAddress; }
            set { Set<string>(() => ServerAddress, ref _serverAddress, value); }
        }

        private string _connectionStatus;
        public string ConnectionStatus
        {
            get { return _connectionStatus; }
            set { Set<string>(() => ConnectionStatus, ref _connectionStatus, value); }
        }

        void OnMessageReceived(object sender, MessageReceivedEventArgs e)
        {
            DispatcherHelper.CheckBeginInvokeOnUI(() =>
                {
                    ConnectionStatus = "Message " + e.MessageType.ToString() + " received from " + e.FromAddress;
                });
        }

        void OnConnectionReceived(object sender, ConnectionReceivedEventArgs e)
        {
            DispatcherHelper.CheckBeginInvokeOnUI(() =>
            {
                ConnectionStatus = "Connection received from " + e.RemoteHost;
            });
        }





        public RelayCommand ConnectCommand
        {
            get;
            private set;
        }

        public void Connect()
        {
            _messageService.Connect(ServerAddress);
            ConnectionStatus = "Connected to server at " + ServerAddress + ".";
        }

        public bool CanConnect()
        {
            return true;
        }




        public RelayCommand ListenCommand
        {
            get;
            private set;
        }

        public void Listen()
        {
            _messageService.Listen();

            var hostNames = NetworkInformation.GetHostNames();

            ConnectionStatus = "Waiting for connection on: ";

            int i = 0;

            foreach (HostName name in hostNames)
            {
                if (i > 0)
                    ConnectionStatus += " and ";

                ConnectionStatus += name.DisplayName;
                i++;
            }

        }

        public bool CanListen()
        {
            return true;
        }


    }
}