﻿using SocketApp.Model;
using System;
using System.Collections.Generic;
using System.Linq;
using Windows.Networking;

namespace SocketApp.Services
{
    public enum WireMessageType
    {
        ChatMessage,
        PlayerJoin,
        PlayerLeave
    }

    public class ChatMessageReceivedEventArgs : EventArgs
    {
        public ChatMessage Message { get; private set; }

        public ChatMessageReceivedEventArgs(ChatMessage message)
        {
            Message = message;
        }
    }

    public class ConnectionReceivedEventArgs : EventArgs
    {
        public Player Player { get; private set; }
        public HostName HostName { get; private set; }

        public ConnectionReceivedEventArgs(Player player, HostName hostName)
        {
            Player = player;
            HostName = hostName;
        }
    }

    public class PlayerJoinedEventArgs : EventArgs
    {
        public Player Player { get; private set; }

        public PlayerJoinedEventArgs(Player player)
        {
            Player = player;
        }
    }

    public class PlayerExitedEventArgs : EventArgs
    {
        public Player Player { get; private set; }

        public PlayerExitedEventArgs(Player player)
        {
            Player = player;
        }
    }

    public interface IMessageService
    {
        void Connect(Player me, string remoteHostName);
        void Disconnect();
        void Listen(Player me);

        void SendChatMessage(ChatMessage message);

        IReadOnlyList<HostName> GetHostNames();

        event EventHandler<ChatMessageReceivedEventArgs> ChatMessageReceived;
        event EventHandler<ConnectionReceivedEventArgs> ConnectionReceived;
        event EventHandler<PlayerJoinedEventArgs> PlayerJoined;
        event EventHandler<PlayerExitedEventArgs> PlayerExited;
    }
}
